// ~~~ OpenGL & Win32 ~~~
// Tutorial No.1 - Template
// Alan Baylis 2001

#include <windows.h>    // Header file for Windows
#include "shared.h"
#include "general.h"
#include "vector.h"
#include "vertex.h"
#include "quat.h"
#include "matrix.h"
#include "texture.h"
#include "locmath.h"    // Included Math functions (specific to OpenGL)
#include "polygon.h"
#include "mmgr.h"
#include "resource.rh"  // Resources header file

//Windows
static HGLRC hRC;       // Permanent Rendering Context
static HDC hDC;         // Private GDI Device Context
HWND hWnd;              // Storage For Window Handle
RECT screen;            // RECT struct for the screen dimensions
PAINTSTRUCT ps;         // PAINTSTRUCT for WM_PAINT message handler

// Input
bool key[256];          // Array used for keyboard input
bool released_key[256]; // Array used for toggled keyboard input

// Math
float pi = 3.141592;
float radian = pi / 180;

// Texture
TEXTURE * texture = new TEXTURE[1];

// Cube polygons
POLYGON * polygon = new POLYGON[12];

void InitGL(int Width, int Height)    // This is called after the main window is created
{
    SetGLProperties();
    SetGLMaterial();
    SetGLLighting();
    SetGLView(Width, Height);
    SetGLScene(polygon);
    SetGLTexture(texture);
}

void ReSizeGLScene(int Width, int Height)
{
    SetGLView(Width, Height);
}

void DrawGLScene(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    DrawGrid();
    DrawCube(polygon, texture);
    DrawSphere();
    DrawCone();
}

LRESULT CALLBACK WndProc( HWND    hWnd,
                          UINT    message,
                          WPARAM  wParam,
                          LPARAM  lParam )
{
    GLuint    PixelFormat;
    static    PIXELFORMATDESCRIPTOR pfd=
    {
        sizeof(PIXELFORMATDESCRIPTOR),   // Size Of This Pixel Format Descriptor
        1,                               // Version Number (?)
        PFD_DRAW_TO_WINDOW |             // Format Must Support Window
        PFD_SUPPORT_OPENGL |             // Format Must Support OpenGL
        PFD_DOUBLEBUFFER,                // Must Support Double Buffering
        PFD_TYPE_RGBA,                   // Request An RGBA Format
        16,                              // Select A 16Bit Color Depth
        0, 0, 0, 0, 0, 0,                // Color Bits Ignored (?)
        0,                               // No Alpha Buffer
        0,                               // Shift Bit Ignored (?)
        0,                               // No Accumulation Buffer
        0, 0, 0, 0,                      // Accumulation Bits Ignored (?)
        16,                              // 16Bit Z-Buffer (Depth Buffer)
        0,                               // No Stencil Buffer
        0,                               // No Auxiliary Buffer (?)
        PFD_MAIN_PLANE,                  // Main Drawing Layer
        0,                               // Reserved (?)
        0, 0, 0                          // Layer Masks Ignored (?)
    };


    switch (message)                // Tells Windows We Want To Check The Message
    {
        case WM_CREATE:
            hDC = GetDC(hWnd);      // Gets A Device Context For The Window

            PixelFormat = ChoosePixelFormat(hDC, &pfd); // Finds The Closest Match To The Pixel Format We Set Above

            if (!PixelFormat)
            {
                MessageBox(0,"Can't Find A Suitable PixelFormat.","Error",MB_OK|MB_ICONERROR);
                PostQuitMessage(0);      // This Sends A 'Message' Telling The Program To Quit
                break;                   // Prevents The Rest Of The Code From Running
            }

            if(!SetPixelFormat(hDC,PixelFormat,&pfd))
            {
                MessageBox(0,"Can't Set The PixelFormat.","Error",MB_OK|MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }

            hRC = wglCreateContext(hDC);
            if(!hRC)
            {
                MessageBox(0,"Can't Create A GL Rendering Context.","Error",MB_OK|MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }

            if(!wglMakeCurrent(hDC, hRC))
            {
                MessageBox(0,"Can't activate GLRC.","Error",MB_OK|MB_ICONERROR);
                PostQuitMessage(0);
                break;
            }
            InitGL(screen.right, screen.bottom);
        break;

        case WM_SYSCOMMAND:                          // Intercept System Commands
        {
            switch (wParam)                          // Check System Calls
            {
                case SC_SCREENSAVE:                  // Screensaver Trying To Start?
                case SC_MONITORPOWER:                // Monitor Trying To Enter Powersave?
                    return 0;                        // Prevent From Happening
            }
            break;                                   // Exit
        }

// This WM_PAINT msg handler will be required when dialog windows are used
/*
        case WM_PAINT:
            BeginPaint(hWnd,&ps);
            DrawGLScene();
            glFlush();
            SwapBuffers(hDC);
            EndPaint(hWnd,&ps);
        break;
*/
        case WM_DESTROY:
        case WM_CLOSE:
            delete[] texture;
            delete[] polygon;
            
            ChangeDisplaySettings(NULL, 0);
            wglMakeCurrent(hDC,NULL);
            wglDeleteContext(hRC);
            ReleaseDC(hWnd,hDC);
            PostQuitMessage(0);
        break;

        case WM_KEYDOWN:
            key[wParam] = TRUE;
        break;

        case WM_KEYUP:
            key[wParam] = FALSE;
        break;

        case WM_SIZE:
            ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));
        break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (0);
}

int WINAPI WinMain( HINSTANCE    hInstance,
                    HINSTANCE    hPrevInstance,
                    LPSTR        lpCmdLine,
                    int          nCmdShow)
{
    MSG        msg;        // Windows Message Structure
    WNDCLASSEX    wc;      // Windows Class Structure Used To Set Up The Type Of Window

    GetWindowRect(GetDesktopWindow(), &screen);
    wc.cbSize           = sizeof(WNDCLASSEX);
    wc.style            = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS | CS_SAVEBITS;
    wc.lpfnWndProc      = (WNDPROC) WndProc;
    wc.cbClsExtra       = 0;
    wc.cbWndExtra       = 0;
    wc.hInstance        = hInstance;
    wc.hIcon            = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MYICON));
    wc.hIconSm          = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MYICON));
    wc.hCursor          = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground    = NULL;
    wc.lpszMenuName     = NULL;
    wc.lpszClassName    = "OpenGL WinClass";


    if(!RegisterClassEx(&wc))
    {
        MessageBox(0,"Failed To Register The Window Class.","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }

    hWnd = CreateWindowEx(
    WS_EX_LEFT,
    "OpenGL WinClass",
    "OpenGL & Win32 Tutorial No.1", // Title Appearing At The Top Of The Window

    WS_MAXIMIZE |
    WS_CLIPCHILDREN |
    WS_CLIPSIBLINGS |
    WS_POPUPWINDOW |
    WS_VISIBLE,
    0, 0,                          // The Position Of The Window On The Screen
    screen.right, screen.bottom,   // The Width And Height Of The WIndow
    NULL,
    NULL,
    hInstance,
    NULL);

    if(!hWnd)
    {
        MessageBox(0,"Window Creation Error.","Error",MB_OK|MB_ICONERROR);
        return FALSE;
    }

    DEVMODE dmScreenSettings;                              // Developer Mode

    memset(&dmScreenSettings, 0, sizeof(DEVMODE));         // Clear Room To Store Settings
    dmScreenSettings.dmSize        = sizeof(DEVMODE);      // Size Of The Devmode Structure
    dmScreenSettings.dmPelsWidth    = screen.right;        // Screen Width
    dmScreenSettings.dmPelsHeight    = screen.bottom;      // Screen Height
    dmScreenSettings.dmFields    = DM_PELSWIDTH | DM_PELSHEIGHT; // Pixel Mode
    ChangeDisplaySettings(&dmScreenSettings, CDS_FULLSCREEN); // Switch To Full Screen

    ShowWindow(hWnd, SW_SHOW);
    UpdateWindow(hWnd);
    SetFocus(hWnd);
    wglMakeCurrent(hDC,hRC);

    while (1)
    {
        // Process All Messages
        while (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
        {
            if (GetMessage(&msg, NULL, 0, 0))
            {
                TranslateMessage(&msg);
                DispatchMessage(&msg);
            }
            else
            {
                return TRUE;
            }
        }

        DrawGLScene();    // Draw The Scene
        glFlush();        // Block the program until cached rendering commands are complete
        SwapBuffers(hDC); // Swap Screen Buffers

        if (key[VK_ESCAPE])
            SendMessage(hWnd,WM_CLOSE,0,0);
    }
}
